@@ -109,7 +109,7 @@ This method must be called before using any other method in this module."
                    (setq keysym (xcb:keysyms:keycode->keysym obj keycode 0)))
           (setq events
                 (nconc events
-                       (list (xcb:keysyms:keysym->event keysym nil t))))))
+                       (list (xcb:keysyms:keysym->event obj keysym nil t))))))
       (cond ((memq 'mode-switch* events)
              (setq xcb:keysyms:mode-switch-mask (elt mode-masks i)))
             ((memq 'kp-numlock events)
@@ -164,29 +164,42 @@ SHIFT LOCK is ignored."
          (group (delq nil (mapcar (lambda (i)
                                     (when (= keycode (car i)) (cdr i)))
                                   keysyms)))
-         (group (pcase (length group)
-                  (1 (append group '(0) group '(0)))
-                  (2 (append group group))
-                  (3 (append group '(0)))
-                  (_
-                   (list (elt group 0) (elt group 1)
-                         (elt group 2) (elt group 3)))))
-         (group (if (and xcb:keysyms:mode-switch-mask ;not initialized
-                         (/= 0
-                             (logand modifiers xcb:keysyms:mode-switch-mask)))
-                    (cddr group) (list (elt group 0) (elt group 1))))
+         (mode-switch-on (and xcb:keysyms:mode-switch-mask ;not initialized
+                              (/= 0 (logand modifiers
+                                            xcb:keysyms:mode-switch-mask))))
          (mask (logior (if (= 0 (logand modifiers xcb:keysyms:shift-mask)) 0 1)
                        (if (= 0 (logand modifiers xcb:keysyms:lock-mask))
                            0 2))))
+    (pcase (length group)
+      (1 (setq group (vector (elt group 0) nil)))
+      (2 (setq group (vector (elt group 0) (elt group 1))))
+      (3 (setq group (if mode-switch-on
+                         (vector (elt group 2) nil)
+                       (vector (elt group 0) (elt group 1)))))
+      (_ (setq group (if mode-switch-on
+                         (vector (elt group 2) (elt group 3))
+                       (vector (elt group 0) (elt group 1))))))
+    (unless (aref group 0)
+      (setq group (vector 0 (aref group 1))))
+    (unless (aref group 1)
+      (setq group (aref group 0)
+            group (if (<= #x20 group #xff)
+                      ;; Only do case conversions for Latin-1 characters
+                      (vector (downcase group) (upcase group))
+                    (vector group group))))
     (if (and xcb:keysyms:num-lock-mask  ;not initialized
              (/= 0 (logand modifiers xcb:keysyms:num-lock-mask))
-             (<= #xff80 (elt group 1)) (>= #xffbe (elt group 1))) ;keypad
-        (if (= mask 1) (elt group 0) (elt group 1))
+             (<= #xff80 (aref group 1) #xffbe)) ;keypad
+        (if (= mask 1) (aref group 0) (aref group 1))
       (pcase mask
-        (0 (elt group 0))               ;SHIFT off, CAPS LOCK off
-        (1 (elt group 1))               ;SHIFT on, CAPS LOCK off
-        (2 (upcase (elt group 0)))      ;SHIFT off, CAPS LOCK on
-        (3 (upcase (elt group 1)))))))  ;SHIFT on, CAPS LOCK on
+        (0 (aref group 0))              ;SHIFT off, CAPS LOCK off
+        (1 (aref group 1))              ;SHIFT on, CAPS LOCK off
+        (2                              ;SHIFT off, CAPS LOCK on
+         (if (<= #x20 (aref group 0) #xff)
+             (upcase (aref group 0)) (aref group 0)))
+        (3                              ;SHIFT on, CAPS LOCK on
+         (if (<= #x20 (aref group 1) #xff)
+             (upcase (aref group 1)) (aref group 1)))))))
 
 (cl-defmethod xcb:keysyms:keysym->keycode ((obj xcb:connection) keysym)
   "Convert X keysym to (first match) keycode"
@@ -317,7 +330,7 @@ SHIFT LOCK is ignored."
         ,@(make-list 64 nil)]
   "Emacs event representations of XF86keysym (#x1008ff00 - #x1008ffff)")
 
-(defun xcb:keysyms:event->keysym (event)
+(cl-defmethod xcb:keysyms:event->keysym ((obj xcb:connection) event)
   "Translate Emacs key event EVENT to X Keysym.
 
 This function returns nil when it fails to convert an event."
@@ -345,43 +358,51 @@ This function returns nil when it fails to convert an event."
                                               xcb:keysyms:-iso-function-keys))
                            ;; ISO function keys
                            (logior keysym #xfe00)))))))
-      (if (and (<= #x20 event) (>= #xff event)) ;Latin-1
+      (if (<= #x20 event #xff)          ;Latin-1
           (setq keysym event)
-        (when (and (<= #x100 event) (>= #x10ffff event)) ;Unicode
+        (when (<= #x100 event #x10ffff) ;Unicode
           (setq keysym (+ #x1000000 event)))))
     (when keysym
-      `(,keysym
-        ;; state for KeyPress event
-        ,(apply #'logior
-                (mapcar (lambda (i)
-                          (pcase i
-                            (`meta xcb:keysyms:meta-mask)
-                            (`control xcb:keysyms:control-mask)
-                            (`shift xcb:keysyms:shift-mask)
-                            (`hyper xcb:keysyms:hyper-mask)
-                            (`super xcb:keysyms:super-mask)
-                            (`alt xcb:keysyms:alt-mask)
-                            (`down 0)
-                            ;; FIXME: more?
-                            (_ 0)))
-                        modifiers))))))
+      (let ((keycode (xcb:keysyms:keysym->keycode obj keysym))
+            (keysyms (plist-get (slot-value obj 'extra-plist) 'keysyms)))
+        (unless (equal keysym (cdr (assoc keycode keysyms)))
+          ;; Shift key is required to input the KEYSYM
+          (cl-pushnew 'shift modifiers)))
+      (setq modifiers
+            (mapcar (lambda (i)
+                      (pcase i
+                        (`meta xcb:keysyms:meta-mask)
+                        (`control xcb:keysyms:control-mask)
+                        (`shift xcb:keysyms:shift-mask)
+                        (`hyper xcb:keysyms:hyper-mask)
+                        (`super xcb:keysyms:super-mask)
+                        (`alt xcb:keysyms:alt-mask)
+                        (`down 0)
+                        ;; FIXME: more?
+                        (_ 0)))
+                    modifiers))
+      (unless (memq nil modifiers)
+        `(,keysym
+          ;; state for KeyPress event
+          ,(apply #'logior modifiers))))))
 
-(defun xcb:keysyms:keysym->event (keysym &optional mask allow-modifiers)
+(cl-defmethod xcb:keysyms:keysym->event ((_obj xcb:connection) keysym
+                                         &optional mask allow-modifiers)
   "Translate X Keysym KEYSYM into Emacs key event.
 
 One may use MASK to provide modifier keys.  If ALLOW-MODIFIERS is non-nil,
 this function will also return symbols for pure modifiers keys."
-  (let ((event (cond ((and (<= #x20 keysym) (>= #xff keysym))
+  (let ((event (cond ((<= #x20 keysym #xff)
                       keysym)
-                     ((and (<= #xff00 keysym) (>= #xffff keysym))
+                     ((<= #xff00 keysym #xffff)
                       (aref xcb:keysyms:-function-keys (logand keysym #xff)))
-                     ((and (<= #x1000100 keysym) (>= #x110ffff keysym))
+                     ((<= #x1000100 keysym #x110ffff)
                       (- keysym #x1000000))
-                     ((and (<= 1 keysym) (>= 5 keysym)) ;ButtonPress assuemd
+                     ((<= 1 keysym 5)   ;ButtonPress assuemd
                       (intern-soft (format "down-mouse-%d" keysym)))
-                     ((and (<= #x1008ff00 keysym) (>= #x1008ffff keysym))
+                     ((<= #x1008ff00 keysym #x1008ffff)
                       (aref xcb:keysyms:-xf86-keys (logand keysym #xff)))
-                     ((and (<= #xfe00 keysym) (>= #xfeff keysym))
+                     ((<= #xfe00 keysym #xfeff)
                       (aref xcb:keysyms:-iso-function-keys
                             (logand keysym #xff))))))
     (when (and (not allow-modifiers)
@@ -400,9 +421,8 @@ this function will also return symbols for pure modifiers keys."
         (when (/= 0 (logand mask xcb:keysyms:control-mask))
           (push 'control event))
         (when (and (/= 0 (logand mask xcb:keysyms:shift-mask))
-                   ;; Emacs only set shift bit for letters
-                   (integerp (car (last event)))
-                   (<= ?A (car (last event))) (>= ?Z (car (last event))))
+                   (or (not (<= #x20 keysym #xff)) ;Not a Latin-1 character
+                       (<= ?A keysym ?Z)))         ;An uppercase letter
           (push 'shift event))
         (when (and xcb:keysyms:hyper-mask
                    (/= 0 (logand mask xcb:keysyms:hyper-mask)))
